{
    hover: {
        animation: {
            duration: 123
        },
        borderColor: '#cccccc',
        brightness: 0.3,
        enabled: true
    },
    inactive: {
        enabled: false
    },
    select: {
        color: '#ff0000',
        enabled: true,
    }
}
