{
    click: function(event) { return true; },
    drag: function(event) { return true; },
    drop: function(event) { return true; },
    mouseOut: function(event) { return true; },
    mouseOver: function(event) { return true; }
}
