{
      attr: {
          'stroke': '#ff0000',
          'stroke-width': 2,
          'rotation': 45,
          'd': ['M', 10, 10, 'L', 30, 30, 'z']
      },
      position: {
          align: 'center',
          verticalAlign: 'top',
          x: 10,
          y: 15
      },
      useHTML: false
}
