{
  data: [
      {
        accessibility: {
            description: 'Some description goes here',
            enabled: true
        },
        className: 'some-class-name',
        color: '#ccc',
        colorIndex: 2,
        custom: {
            'some_key': 123,
            'other_key': 456
        },
        description: 'Some description goes here',
        events: {
          click: function(event) { return true; },
          drag: function(event) { return true; },
          drop: function(event) { return true; },
          mouseOut: function(event) { return true; }
        },
        id: 'some-id-goes-here',
        labelrank: 3,
        name: 'Some Name Goes here',
        selected: false
      },
      {
        accessibility: {
            description: 'Some description goes here',
            enabled: true
        },
        className: 'some-class-name',
        color: '#ccc',
        colorIndex: 2,
        custom: {
            'some_key': 123,
            'other_key': 456
        },
        description: 'Some description goes here',
        events: {
          click: function(event) { return true; },
          drag: function(event) { return true; },
          drop: function(event) { return true; },
          mouseOut: function(event) { return true; }
        },
        id: 'some-id-goes-here',
        labelrank: 3,
        name: 'Some Name Goes here',
        selected: false
      }
  ],
  id: 'some-id-goes-here',
  index: 3,
  legendIndex: 3,
  name: 'Series Name Goes Here',
  stack: 'stack-id',
  xAxis: 'some-id',
  yAxis: 0,
  zIndex: 3
}
