{
    alpha: 0.5,
    axisLabelPosition: 'auto',
    beta: 15,
    depth: 20,
    enabled: true,
    fitToPlot: true,
    frame: {
        back: {
            color: '#ccc',
            size: 2,
            visible: 'auto'
        },
        bottom: {
            color: '#ccc',
            size: 2,
            visible: 'auto'
        },
        front: {
            color: '#ccc',
            size: 2,
            visible: 'auto'
        },
        left: {
            color: '#ccc',
            size: 2,
            visible: 'auto'
        },
        right: {
            color: '#ccc',
            size: 2,
            visible: 'auto'
        },
        size: 2,
        top: {
            color: '#ccc',
            size: 2,
            visible: 'auto'
        },
        visible: 'default'
    },
    viewDistance: 20
}
