{
    className: 'CircleAnnotationBinding',
    init: 'some-event-marker',
    start: 'some-event-marker',
    steps: [
        'some-event-marker',
        'some-event-marker',
        'some-event-marker'
    ],
    end: 123
  }
