{
    afterBreaks: function(event) { return true; },
    afterSetExtremes: 'not a valid function',
    pointBreak: function(event) { return true; },
    setExtremes: function(event) { return true; }
}
