{
    allowForce: true,
    debug: {
        showSkipSummary: false,
        timeBufferCopy: true,
        timeKDTree: true,
        timeRendering: false,
        timeSeriesProcessing: true,
        timeSetup: true
    },
    enabled: true,
    pixelRatio: 'not a valid value',
    seriesThreshold: 5000,
    useGPUTranslations: true,
    usePreallocated: true
}
