{
   item1: {
       onclick: """function() { return true; }""",
       text: 'My Menu Item',
       textKey: 'my-menu-item',
       separator: False
   },
   item2: {
       onclick: """function() { return true; }""",
       text: 'Second Menu Item',
       textKey: 'my-menu-item2',
       separator: False
   }
}
