{
    circleAnnotation: {
      init: 'some-event-marker',
      start: 'some-event-marker',
      steps: [
          'some-event-marker',
          'some-event-marker',
          'some-event-marker'
      ],
      end: 'some-event-marker'
    },
    ellipseAnnotation: {
      init: 'some-event-marker',
      start: 'some-event-marker',
      steps: [
          'some-event-marker',
          'some-event-marker'
      ],
      end: 'some-event-marker'
    },
    labelAnnotation: {
      init: 'some-event-marker',
      start: 'some-event-marker',
      end: 'some-event-marker'
    },
    rectangleAnnotation: {
      init: 'some-event-marker',
      start: 'some-event-marker',
      steps: [
          'some-event-marker'
      ],
      end: 'some-event-marker'
    }
}
