{
    annotations: {
        itemDelimiter: '; ',
        join: true
    },
    columnHeaderFormatter: function () { return true; },
    dateFormat: 'format string',
    decimalPoint: '.',
    itemDelimiter: ',',
    lineDelimiter: `
`
}
