{
  data: [
    {
      isIntermediateSum: true,
      isSum: true
    },
    {
      isIntermediateSum: true,
      isSum: true
    }
  ],
  lineColor: '#fff',
  upColor: '#999',

  depth: 10,
  edgeColor: '#999',
  edgeWidth: 1,
  groupZPadding: 4
}
