{
    buttonSpacing: 6,
    buttonTheme: {
        fill: '#fff'
    },
    events: {
      click: function(event) { return true; }
    },
    floating: true,
    format: 'some format string',
    formatter: function () { return true; },
    position: undefined,
    relativeTo: 'invalid value',
    rtl: false,
    separator: {
        text: '>',
        style: {
            some-key: 'some-value'
        }
    },
    useHTML: false,
    zIndex: 3
  }
