{
  rules: [
    {
      condition: {
        maxHeight: 20,
        maxWidth: 30,
        minHeight: 10,
        minWidth: 10
      }
    },
    {
      condition: {
        maxHeight: 123.45,
        maxWidth: 543.21,
        minHeight: 678.9,
        minWidth: 321
      }
    }
  ]
}
