{
  data: [
      'invalid value',
      {
        dataLabels: {
          align: 'center',
          allowOverlap: true,
          animation: {
              defer: 5
          },
          backgroundColor: {
              linearGradient: {
                  x1: 0.123,
                  x2: 0.234,
                  y1: 0.345,
                  y2: 0.456
              },
              stops: [
                  [0.12, '#999'],
                  [0.34, '#fff']
              ]
          },
          borderColor: '#999999',
          borderRadius: 24,
          borderWidth: 1,
          className: 'some-class-name',
          color: '#000000',
          crop: true,
          defer: false,
          enabled: true,
          filter: {
              operator: '>=',
              property: 'some_property',
              value: 123
          },
          format: 'some format',
          formatter: function() { return true; },
          inside: true,
          nullFormat: 'some format',
          nullFormatter: function() { return true; },
          overflow: 'none',
          padding: 12,
          position: 'center',
          rotation: 0,
          shadow: false,
          shape: 'rect',
          style: 'style goes here',
          useHTML: false,
          verticalAlign: 'top',
          x: 10,
          y: 20,
          z: 0
        },
        dragDrop: {
            draggableX: true,
            draggableY: true,
            dragHandle: {
                className: 'draghandle-classname-goes-here',
                color: '#ccc',
                cursor: 'alias',
                lineColor: '#ddd',
                lineWidth: 2,
                pathFormatter: function() { return true; },
                zIndex: 10
            },
            dragMaxX: 3456,
            dragMaxY: 6532,
            dragMinX: 123,
            dragMinY: 321,
            dragPrecisionX: 5,
            dragPrecisionY: 5,
            dragSensitivity: 2,
            groupBy: 'some-property-name',
            guideBox: {
                default: {
                    className: 'some-classname-goes-here',
                    color: '#999',
                    cursor: 'pointer',
                    lineColor: '#ccc',
                    lineWidth: 2,
                    zIndex: 100
                }
            },
            liveRedraw: true
        },
        drilldown: 'some-id-goes-here',
        marker: {
          enabled: true,
          fillColor: '#cccccc',
          height: 24,
          lineWidth: 2,
          radius: 2,
          states: {
              hover: {
                  enabled: true
              }
          },
          symbol: 'circle',
          width: 48
        },
        x: 1658808240,
        y: 123
      },
      {
        dataLabels: {
          align: 'center',
          allowOverlap: true,
          animation: {
              defer: 5
          },
          backgroundColor: {
              linearGradient: {
                  x1: 0.123,
                  x2: 0.234,
                  y1: 0.345,
                  y2: 0.456
              },
              stops: [
                  [0.12, '#999'],
                  [0.34, '#fff']
              ]
          },
          borderColor: '#999999',
          borderRadius: 24,
          borderWidth: 1,
          className: 'some-class-name',
          color: '#000000',
          crop: true,
          defer: false,
          enabled: true,
          filter: {
              operator: '>=',
              property: 'some_property',
              value: 123
          },
          format: 'some format',
          formatter: function() { return true; },
          inside: true,
          nullFormat: 'some format',
          nullFormatter: function() { return true; },
          overflow: 'none',
          padding: 12,
          position: 'center',
          rotation: 0,
          shadow: false,
          shape: 'rect',
          style: 'style goes here',
          useHTML: false,
          verticalAlign: 'top',
          x: 10,
          y: 20,
          z: 0
        },
        dragDrop: {
            draggableX: true,
            draggableY: true,
            dragHandle: {
                className: 'draghandle-classname-goes-here',
                color: '#ccc',
                cursor: 'alias',
                lineColor: '#ddd',
                lineWidth: 2,
                pathFormatter: function() { return true; },
                zIndex: 10
            },
            dragMaxX: 3456,
            dragMaxY: 6532,
            dragMinX: 123,
            dragMinY: 321,
            dragPrecisionX: 5,
            dragPrecisionY: 5,
            dragSensitivity: 2,
            groupBy: 'some-property-name',
            guideBox: {
                default: {
                    className: 'some-classname-goes-here',
                    color: '#999',
                    cursor: 'pointer',
                    lineColor: '#ccc',
                    lineWidth: 2,
                    zIndex: 100
                }
            },
            liveRedraw: true
        },
        drilldown: 'some-id-goes-here',
        marker: {
          enabled: true,
          fillColor: '#cccccc',
          height: 24,
          lineWidth: 2,
          radius: 2,
          states: {
              hover: {
                  enabled: true
              }
          },
          symbol: 'circle',
          width: 48
        },
        x: Date.UTC(2022, 6, 26),
        y: 123
      }
  ],
  id: 'some-id-goes-here',
  index: 3,
  legendIndex: 3,
  name: 'Series Name Goes Here',
  stack: 'stack-id',
  xAxis: 'some-id',
  yAxis: 0,
  zIndex: 3
}
