{
  column: 'not a nmber',
  level: 2,
  color: '#cccccc',
  colorIndex: 123,
  dataLabels: {
    align: 'center',
    allowOverlap: false,
    backgroundColor: '#cccccc',
    enabled: true,
    overflow: 'justify',
    shadow: false
  },
  id: 'some-id-goes-here',
  name: 'My Node Name',
  offsetHorizontal: 10,
  offsetVertical: '5%'
}
