{
    color: '#cccccc',
    colorIndex: 'not a number',
    dataLabels: {
        align: 'center',
        allowOverlap: false,
        backgroundColor: '#cccccc',
        enabled: true,
        overflow: 'justify',
        shadow: false
    },
    id: 'some-id-goes-here',
    name: 'My Node Name',
    offsetHorizontal: 10,
    offsetVertical: 'not a valid number or % string'
}
