{
    animation: {
        defer: 5
    },
    patternOptions: {
        aspectRatio: 0.5,
        backgroundColor: '#999999',
        id: 'some_id_goes_here',
        opacity: 0.5,
        width: 120,
        x: 5,
        y: 10
    },
    patternIndex: 2
}
