{
    contextButtonTitle: 'some title goes here',
    decimalPoint: '--',
    downloadCSV: 'Download CSV',
    downloadJPEG: 'Download JPEG',
    downloadPDF: 'Download PDF',
    downloadPNG: 'Download PNG',
    downloadSVG: 'Download SVG',
    downloadXLS: 'Download Excel',
    drillUpText: 'Backup',
    exitFullscreen: 'Exit Fullscreen Mode',
    hideData: 'Hide',
    invalidDate: 'Invalid Time Period',
    loading: 'Loading...',
    mainBreadcrumb: 'Home',
    months: [
        'Jan',
        'Feb',
        'Mar',
        'Apr',
        'May',
        'Jun',
        'Jul',
        'Aug',
        'Sep',
        'Oct',
        'Nov',
        'Dec'
    ],
    noData: 'No data available.',
    numericSymbolMagnitude: 1,
    numericSymbols: ['#', 'k', 'm'],
    printChart: 'Print',
    resetZoom: 'Reset',
    resetZoomTitle: 'Reset Zoom',
    shortMonths: 'invalid value',
    shortWeekdays: [
        'Mon',
        'Tue',
        'Wed',
        'Thurs',
        'Fri',
        'Sat',
        'Sun'
    ],
    thousandsSep: ',',
    viewData: 'View Data',
    viewFullscreen: 'Fullscreen',
    weekdays: [
        'Mon',
        'Tue',
        'Wed',
        'Thurs',
        'Fri',
        'Sat',
        'Sun'
    ]
}
