{
      background: [{
          className: 'test-class-name',
          innerRadius: '24%',
          shape: 'circle'
      }],
      center: ['50%', '30%', '75%'],
      size: 120,
      startAngle: 0
}
