{
  allowExtendPlayingField: false,
  animationLimit: 10,
  borderColor: '#ccc',
  borderRadius: 'invalid value',
  borderWidth: 1,
  centerInCategory: true,
  colorByPoint: false,
  colorIndex: 2,
  colorKey: 'some-key-goes-here',
  colors: [
      '#ccc',
      '#fff',
      '000000'
  ],
  edgeWidth: 1,
  maxFontSize: 36,
  minFontSize: 6,
  placementStrategy: 'center',
  rotation: {
      from: 0,
      orientations: 4,
      to: 135
  },
  spiral: 'rectangle'
}
