{
    announceNewData: {
        newDataAnnounce: 'New Data!',
        newPointAnnounceMultiple: 'New data points added',
        newPointAnnounceSingle: 'New data point added',
        newSeriesAnnounceMultiple: 'New data series added',
        newSeriesAnnounceSingle: 'Added a data series'
    },
    axis: {
        rangeCategories: 'Categories',
        rangeFromTo: 'X to Y',
        timeRangeDays: 'Monday - Friday',
        timeRangeHours: '0h - 24h',
        timeRangeMinutes: ':00 - :59',
        timeRangeSeconds: ':00:00 - :00:59',
        xAxisDescriptionPlural: 'X Axis Description when pluralized',
        xAxisDescriptionSingular: 'X Axis Description',
        yAxisDescriptionPlural: 'Y-Axis description when pluralized',
        yAxisDescriptionSingular: 'Y-Axis Description'
    },
    chartContainerLabel: 'some string',
    chartTypes: {
        barMultiple: 'Bar Charts',
        barSingle: 'Bar Chart',
        boxplotMultiple: 'Boxplot Charts',
        boxplotSingle: 'Boxplot Chart',
        bubbleMultiple: 'Bubble Charts',
        bubbleSingle: 'Bubble Chart',
        columnMultiple: 'Column Charts',
        columnSingle: 'Column Chart',
        combinationChart: 'Combo Chart',
        defaultMultiple: 'Charts',
        defaultSingle: 'Chart',
        emptyChart: 'Empty Chart',
        lineMultiple: 'Line Charts',
        lineSingle: 'Line Chart',
        mapTypeDescription: 'Map',
        pieMultiple: 'Pie Charts',
        pieSingle: 'Pie Chart',
        scatterMultiple: 'Scatter Charts',
        scatterSingle: 'Scatter Chart',
        splineMultiple: 'Spline Charts',
        splineSingle: 'Spline Chart',
        unknownMap: 'Unknown Map'
    },
    credits: 'credits go here',
    defaultChartTitle: 'default title goes here',
    drillUpButton: 'drillup button label',
    exporting: {
        chartMenuLabel: 'Menu',
        menuButtonLabel: 'Menu Button'
    },
    graphicContainerLabel: 'Graphic Container',
    legend: {
        legendItem: 'Item',
        legendLabel: 'Legend',
        legendLabelNoTitle: 'Label with no Title'
    },
    rangeSelector: {
        clickButtonAnnouncement: 'Selected Range',
        dropdownLabel: 'Range',
        maxInputLabel: 'Max.',
        minInputLabel: 'Min.'
    },
    screenReaderSection: {
        afterRegionLabel: 'After Region',
        annotations: {
            descriptionMultiplePoints: 'Description of multiple points',
            descriptionNoPoints: 'Description with no points',
            descriptionSinglePoint: 'Description of a single point',
            heading: 'My Heading'
        },
        beforeRegionLabel: 'Before Region',
        endOfChartMarker: 'End of Chart'
    },
    sonification: {
        playAsSoundButtonText: 'Play as Sound',
        playAsSoundClickAnnouncement: 'Playing as Sound'
    },
    svgContainerLabel: 'SVG',
    svgContainerTitle: 'SVG',
    table: {
        tableSummary: 'Summary goes here',
        viewAsDataTableButtonText: 'View as Table'
    },
    thousandsSep: ',',
    zoom: {
        mapZoomIn: 'Zoom In',
        mapZoomOut: 'Zoom out',
        resetZoomButton: 'Reset Zoom'
    }
  }
