{
  fillColor: '#ccc',
  fillOpacity: 0.7,
  lineColor: {
      radialGradient: {
          cx: 0.123,
          cy: 0.456,
          r: 0.789
      },
      stops: [
          [0.123, '#cccccc'],
          [0.456, '#ff0000'],
          [1, '#00ff00']
      ]
  },
  negativeFillColor: 123,
  trackByArea: true
}
