{
    accessibility: {
        enabled: true,
        keyboardNavigation: {
            enabled: true
        }
    },
    align: 'right',
    alignColumns: true,
    backgroundColor: '#fff',
    borderColor: '#ccc',
    borderRadius: 4,
    className: 'some-class-name',
    enabled: true,
    floating: false,
    itemCheckboxStyle: 'some-style-setting',
    itemDistance: 12,
    itemHiddenStyle: 'some-style-setting',
    itemHoverStyle: 'some-style-setting',
    itemMarginBottom: 7,
    itemMarginTop: 7,
    itemStyle: 'some-style-setting',
    itemWidth: 36,
    labelFormat: 'format string',
    labelFormatter: function () { return true; },
    layout: 'vertical',
    margin: 7,
    maxHeight: 120,
    padding: 8,
    reversed: false,
    rtl: false,
    shadow: false,
    squareSymbol: false,
    symbolHeight: 12,
    symbolPadding: 2,
    symbolRadius: 0,
    symbolWidth: 12,
    title: {
        style: 'some-style-string',
        text: 'Legend Title Goes Here'
    },
    useHTML: false,
    verticalAlign: 'top',
    width: 120,
    x: 0,
    y: 0
  }
