{
    borderColor: '#ccc',
    borderWidth: 12,
    className: 'some-class-name',
    color: '#fff',
    connectorClassName: 'some-class-name',
    connectorColor: '#999',
    connectorDistance: 6,
    connectorWidth: 2,
    enabled: true,
    labels: {
        align: 'center',
        allowOverlap: false,
        className: 'some-class-name',
        format: 'a format string',
        formatter: function () { return true; },
        style: 'some style string',
        x: 3,
        y: 1
    },
    legendIndex: 1,
    maxSize: 24,
    minSize: 6,
    ranges: [
        {
         borderColor: '#ccc',
         color: '#fff',
         connectorColor: '#999',
         value: 123
        },
        {
         borderColor: '#ccc',
         color: '#fff',
         connectorColor: '#999',
         value: 123
        },
        {
         borderColor: '#ccc',
         color: '#fff',
         connectorColor: '#999',
         value: 123
        }
    ],
    sizeBy: 'width',
    sizeByAbsoluteValue: true,
    zIndex: 3,
    zThreshold: 123
  }
