{
  accessibility: {
      description: 'Some description goes here',
      enabled: true
  },
  className: 'some-class-name',
  color: '#ccc',
  colorIndex: 2,
  custom: 'invalid value',
  description: 'Some description goes here',
  events: {
    click: function(event) { return true; },
    drag: function(event) { return true; },
    drop: function(event) { return true; },
    mouseOut: function(event) { return true; }
  },
  id: 'some-id-goes-here',
  labelRank: 3,
  name: 'Some Name Goes here',
  selected: false
}
