{
    className: 'some-class-name',
    color: '#fff',
    dashStyle: 'Dash',
    events: {
      click: function(event) { return true; },
      mousemove: function(event) { return true; },
      mouseout: function(event) { return true; },
      mouseover: function(event) { return true; }
    },
    label: {
        align: 'center',
        formatter: function() { return true; },
        rotation: 0,
        style: 'some-style-string',
        text: 'The title of the plot band',
        textAlign: 'left',
        useHTML: false,
        verticalAlign: 'middle',
        x: -10,
        y: 100
    },
    value: 123,
    width: 2,
    zIndex: 3
  }
