from highcharts_core.options.plot_options.series import SeriesOptions


class ParetoOptions(SeriesOptions):
    """General options to apply to all Pareto series types.

    A pareto diagram is a type of chart that contains both bars and a line graph,
    where individual values are represented in descending order by bars, and the
    cumulative total is represented by the line.

    .. figure:: ../../../_static/pareto-example.png
      :alt: Pareto Example Chart
      :align: center

    """
    pass
