from highcharts_core.options.plot_options.series import SeriesOptions


class SplineOptions(SeriesOptions):
    """General options to apply to all Spline series types.

    A spline series is a special type of line series, where the segments between the
    data points are smoothed.

    .. figure:: ../../../_static/spline-example.png
      :alt: Spline Example Chart
      :align: center

    """
    pass
