{
    annotations: {
        itemDelimiter: '; ',
        join: True
    },
    columnHeaderFormatter: function () { return true; },
    dateFormat: 123,
    decimalPoint: '.',
    itemDelimiter: ',',
    lineDelimiter: '\n'
}
