{
    className: 'some-class-name',
    enabled: true,
    menuClassName: 'menu-class',
    menuItems: ['item1', 'item2'],
    onclick: 'not a valid function',
    symbol: 'menu',
    symbolFill: '#ccc',
    text: 'Button Label',
    theme: {
        fill: '#fff',
        stroke: '#ccc'
    },
    titleKey: 'somevalue',
    x: 10,
    y: 0
  }
