{
    activeAxisLabelStyle: 'not a valid style dict',
    activeDataLabelStyle: {
        'cursor': 'style string',
        'color': '#999'
    },
    allowPointDrilldown: true,
    animation: false,
    breadcrumbs: {
      buttonSpacing: 6,
      buttonTheme: {
          fill: '#fff'
      },
      events: {
        click: function(event) { return true; }
      },
      floating: True,
      format: 'some format string',
      formatter: function () { return true; },
      relativeTo: 'plot',
      rtl: False,
      separator: {
          style: {
              'some-key': 'some-value'
          },
          text: '>',
      },
      useHTML: False,
      zIndex: 3
    }
  }
