{
    popup: {
      addButton: 123,
      algorithm: 'some string',
      arrowInfinityLine: 'some string',
      arrowRay: 'some string',
      arrowSegment: 'string',
      average: 'avg',
      background: 'background',
      backgroundColor: 'some string',
      borderRadius: 'some string',
      borderWidth: 'width',
      bottomBand: 'band',
      circle: 'some string',
      clearFilter: 'clear',
      color: 'some string',
      connector: 'some string',
      crooked3: 'some string',
      crooked5: 'some string',
      crosshairX: 'some string',
      crosshairY: 'some string',
      decimals: 'some string',
      deviation: 'some string',
      editButton: 'some string',
      elliott3: 'some string',
      elliott5: 'some string',
      ellipse: 'some string',
      factor: 'some string',
      fastAvgPeriod: 'some string',
      fibonacci: 'some string',
      fibonacciTimeZones: 'some string',
      fill: 'some string',
      flags: 'some string',
      fontSize: 'some string',
      format: 'some string',
      height: 'some string',
      highIndex: 'some string',
      horizontalLine: 'some string',
      increment: 'some string',
      index: 'some string',
      infinityLine: 'some string',
      initialAccelerationFactor: 'some string',
      innerBackground: 'some string',
      label: 'some string',
      labelOptions: 'some string',
      labels: 'some string',
      line: 'some string',
      lines: 'some string',
      longPeriod: 'some string',
      lowIndex: 'some string',
      maxAccelerationFactor: 'some string',
      measure: 'some string',
      measureX: 'some string',
      measureXY: 'some string',
      measureY: 'some string',
      multiplier: 'some string',
      multiplierATR: 'some string',
      name: 'some string',
      noFilterMatch: 'some string',
      outerBackground: 'some string',
      padding: 'some string',
      parallelChannel: 'some string',
      period: 'some string',
      periodATR: 'some string',
      periods: 'some string',
      periodSenkouSpanB: 'some string',
      periodTenkan: 'some string',
      pitchfork: 'some string',
      ranges: 'some string',
      ray: 'some string',
      rectangle: 'some string',
      removeButton: 'some string',
      saveButton: 'some string',
      searchIndicators: 'some string',
      segment: 'some string',
      series: 'some string',
      shapeOptions: 'some string',
      shapes: 'some string',
      shortPeriod: 'some string',
      signalPeriod: 'some string',
      simpleShapes: 'some string',
      slowAvgPeriod: 'some string',
      standardDeviation: 'some string',
      stroke: 'some string',
      strokeWidth: 'some string',
      style: 'some string',
      timeCycles: 'some string',
      title: 'some string',
      topBand: 'some string',
      tunnel: 'some string',
      typeOptions: 'some string',
      verticalArrow: 'some string',
      verticalCounter: 'some string',
      verticalLabel: 'some string',
      verticalLine: 'some string',
      volume: 'some string',
      xAxisUnit: 'some string'
    }
  }
