{
  background: [
      {
        className: 'test-class-name',
        innerRadius: '24%',
        shape: 'circle'
      },
      {
        innerRadius: '35%',
      }
  ],
  center: ['50%', '50%'],
  size: 120,
  startAngle: 0
}
