{
    'color': '#cccccc',
    'color_index': 123,
    'column': 'not a number',
    'data_labels': {
        'align': 'center',
        'allowOverlap': False,
        'backgroundColor': '#cccccc',
        'enabled': True,
        'overflow': 'justify',
        'shadow': False
    },
    'id': 'some-id-goes-here',
    'image': 'http://www.somewhere.com/',
    'layout': 'normal',
    'level': 2,
    'name': 'My Node Name',
    'offset_horizontal': 10,
    'offset_vertical': '5%',
    'title': 123
}
