{
    crosshair: {
        className: 'some-class-name',
        color: '#ccc',
        dashStyle: 'Dash',
        snap: true,
        width: 40,
        zIndex: 6
    },
    height: 60,
    left: 5,
    lineColor: '#999',
    lineWidth: 1,
    showEmpty: true,
    top: 340,
    width: 300
}
