{
   className: 'classname-1',
   from: 1,
   marker: {
     enabled: true,
     fillColor: '#cccccc',
     height: 24,
     lineWidth: 2,
     radius: 2,
     states: {
         hover: {
             enabled: true
         }
     },
     symbol: 'circle',
     width: 48
   },
   to: 123
}
