{
  colorIndex: 3,
  crisp: true,
  dial: {
      backgroundColor: '#ccc',
      baseLength: '80%',
      baseWidth: 120,
      borderColor: '#fff',
      borderWidth: 1,
      path: [1, 2, 3],
      radius: '80%',
      rearLength: '10%',
      topWidth: 80
  },
  linecap: 'round',
  lineWidth: 1,
  overshoot: 12.5,
  pivot: {
      backgroundColor: '#ccc',
      borderColor: '#eee',
      borderWidth: 2,
      radius: 60
  },
  pointInterval: 5,
  pointIntervalUnit: 'day',
  pointStart: 5,
  relativeXValue: true,
  shadow: false,
  wrap: false
}
