{
    anchor: 'start',
    approximation: 'windbarb',
    dateTimeLabelFormats: {
      day: 'test',
      hour: 'test',
      millisecond: 'test',
      minute: 'test',
      month: 'test',
      second: 'test',
      week: 'test',
      year: 'test'
    },
    enabled: true,
    firstAnchor: 'start',
    forced: true,
    groupAll: true,
    groupPixelWidth: 10,
    lastAnchor: 'end',
    units: [
        [
            'millisecond',
            [1, 2, 5, 10, 20, 25, 50, 100, 200, 500]
        ],
        [
            'second',
            [1, 2, 5, 10, 15, 30]
        ],
        [
            'minute',
            [1, 2, 5, 10, 15, 30]
        ],
        [
            'hour',
            [1, 2, 3, 4, 6, 8, 12]
        ],
        [
            'day',
            [1]
        ],
        [
            'week',
            [1]
        ],
        [
            'month',
            [1, 3, 6]
        ],
        [
            'year',
            null
        ]
    ]
}
