{
    arearange: 123,
    areasplinerange: 'Area Spline Range',
    boxplot: 'Boxplot',
    bubble: 'Bubble',
    columnrange: 'Column Range',
    errorbar: 'Errorbar',
    funnel: 'Funnel',
    pyramid: 'Pyramid',
    waterfall: 'Waterfall'
}
